@@ -22,6 +22,8 @@
 
 #include "guile-gtk.h"
 #include "guile-compat.h"
+#include "gtk-types.h"
+#include "gtk-compat.h"
 
 /* GtkObjects.
 
@@ -125,7 +127,12 @@ make_gtkobj (GtkObject *obj)
 SCM
 sgtk_wrap_gtkobj (GtkObject *obj)
 {
-  SCM handle = (SCM) gtk_object_get_data (obj, handle_key);
+  SCM handle;
+
+  if (obj == NULL)
+    return SCM_BOOL_F;
+
+  handle = (SCM) gtk_object_get_data (obj, handle_key);
   if (handle == (SCM) NULL)
     handle = make_gtkobj (obj);
   return handle;
@@ -429,6 +436,9 @@ sgtk_boxed2scm (gpointer ptr, sgtk_boxed_info *info, int copyp)
   SCM z;
   boxed *b;
 
+  if (ptr == NULL)
+    return SCM_BOOL_F;
+
   SCM_DEFER_INTS;
   b = new_boxed ();
   if (copyp)
@@ -611,20 +621,58 @@ sgtk_callback_destroy (gpointer data)
 
 
 
-/* Misc functions and initialization */
-
-/* This is not automatically stubbed so that we can avoid deferring
-   interrupts. */
-
-SCM_PROC (s_gtk_main, "gtk-main", 0, 0, 0, sgtk_main);
+/* Type conversions */
 
 SCM
-sgtk_main ()
+sgtk_color_conversion (SCM color)
 {
-  gtk_main ();
-  return SCM_UNSPECIFIED;
+  SCM orig_color = color;
+
+  if (SCM_NIMP (color) && SCM_STRINGP (color))
+    {
+      GdkColor colstruct;
+      GdkColormap *colmap;
+
+      SCM_DEFER_INTS;
+      if (!gdk_color_parse (SCM_CHARS (color), &colstruct))
+	{
+	  SCM_ALLOW_INTS;
+	  scm_misc_error ("string->color", "no such color: %S",
+			  scm_cons (orig_color, SCM_EOL));
+	}
+      colmap = gtk_widget_peek_colormap ();
+      if (!gdk_color_alloc (colmap, &colstruct))
+	{
+	  SCM_ALLOW_INTS;
+	  scm_misc_error ("string->color", "can't allocate color: %S",
+			  scm_cons (orig_color, SCM_EOL));
+	}
+      SCM_ALLOW_INTS;
+      return sgtk_boxed2scm (&colstruct, &sgtk_gdk_color_info, 1);
+    }
+  return color;
 }
 
+extern SCM sgtk_gdk_font_load (SCM font);
+
+SCM
+sgtk_font_conversion (SCM font)
+{
+  SCM orig_font = font;
+
+  if (SCM_NIMP (font) && SCM_STRINGP (font))
+    {
+      font = sgtk_gdk_font_load (font);
+      if (font == SCM_BOOL_F)
+	scm_misc_error ("string->font", "no such font: %S",
+			scm_cons (orig_font, SCM_EOL));
+    }
+  return font;
+}
+
+
+
+/* Initialization */
 
 extern void sgtk_init_gtk_defs ();
 
