@@ -1,7 +1,7 @@
 /*
  * Configuration-File Functions.
  *
- *  Copyright 1993, 1994, 1997 The G_Free Software Foundation
+ *  Copyright 1993, 1994, 1997 The Free Software Foundation
  *
  * Authors: Miguel de Icaza
 
@@ -138,7 +138,7 @@ parse_path (char *path)
 	
 	p->file    = strtok (p->path, sep);
 	p->section = strtok (NULL, "/=");
-	p->key     = strtok (NULL, "/=");
+	p->key     = strtok (NULL, "=");
 	p->def     = strtok (NULL, "=");
 
 	/* Was it a Gnome-relative pathname? */
@@ -340,7 +340,8 @@ dump_keys (FILE *profile, TKeys *p)
 	if (!p)
 		return;
 	dump_keys (profile, p->link);
-	fprintf (profile, "%s=%s\n", p->key_name, p->value);
+	if (*p->key_name) 
+		fprintf (profile, "%s=%s\n", p->key_name, p->value);
 }
 
 static void 
@@ -473,8 +474,8 @@ gnome_config_iterator_next (void *s, char **key, char **value)
 	TKeys *keys = (TKeys *) s;
 
 	if (keys){
-		*key   = keys->key_name;
-		*value = keys->value;
+		*key   = g_strdup (keys->key_name);
+		*value = g_strdup (keys->value);
 		keys   = keys->link;
 	}
 	return keys;
@@ -506,6 +507,34 @@ gnome_config_clean_section (char *path)
 	release_path (pp);
 }
 
+void 
+gnome_config_clean_key (char *path)
+	/* *section_name, char *file */
+{
+	TSecHeader *section;
+	TKeys *key;
+	ParsedPath *pp;
+	
+	pp = parse_path (path);
+	
+	/* We assume the user has called one of the other initialization funcs */
+	if (!is_loaded (pp->file, &section)){
+		fprintf (stderr,"Warning: profile_clean_section called before init\n");
+		release_path (pp);
+		return;
+	}
+	for (; section; section = section->link){
+		if (strcasecmp (section->section_name, pp->section))
+			continue;
+		for (key = section->keys; key; key = key->link){
+			if (strcasecmp (key->key_name, pp->key))
+				continue;
+			key->key_name [0] = 0;
+		}
+	}
+	release_path (pp);
+}
+
 int 
 gnome_config_has_section (char *path)
 	/* char *section_name, char *profile */
