@@ -9,13 +9,13 @@ struct SelectInfo;
 struct ConfigScreenSaver {
 
      // list of known screensavers
-     GList *ssavers;
+     GHashTable *ssavers;
 
      // current(selected) screensaver mode
      ScreenSaverMode *curMode;
 
      // some important widgets
-     GtkWidget *vbox, *monitor, *mlist;
+     GtkWidget *vbox, *monitor, *mlist, *sw;
 
      ConfigScreenSaver (GtkWidget *);
 
