@@ -6,13 +6,17 @@
 
 #include <gdk/gdkprivate.h>
 #include "xlockmore.h"
+#include "gnome-desktop.h"
 #include <stdarg.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
+#include <libintl.h>
 
 #include <sys/wait.h>
 
+#define _(String) gettext(String)
+
 extern FILE *yyin, *yyout;
 extern int yylex ();
 
@@ -121,12 +125,20 @@ testIt ()
 	printf ("test It !!!\n");
 }
 
-static void
-destroySetupWin (GtkWidget *w, XLockMore *xm)
+static gint
+deleteSetupWin (GtkWidget *w, GdkEvent *, XLockMore *xm)
 {
 	xm->kill (&xm->sPID);
 	xm->setupWin = NULL;
 	xm->mapSignal = xm->unmapSignal = -1;
+
+	return TRUE;
+}
+
+static gint
+destroySetupWin (GtkWidget *w, XLockMore *xm)
+{
+	deleteSetupWin (w, NULL, xm);
 }
 
 extern GtkWidget *main_window;
@@ -136,9 +148,11 @@ XLockMore::prepareSetupWindow ()
 {
 	if (!setupWin) {
 		setupWin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		gtk_signal_connect (GTK_OBJECT (setupWin), "destroy",
-				    GTK_SIGNAL_FUNC (destroySetupWin), this);
-
+		gtk_window_set_policy (GTK_WINDOW (setupWin), FALSE, FALSE, TRUE);
+		gtk_signal_connect (GTK_OBJECT (setupWin), "delete_event",
+				    GTK_SIGNAL_FUNC (deleteSetupWin), this);
+		gtk_signal_connect (GTK_OBJECT (main_window), "delete_event",
+				    GTK_SIGNAL_FUNC (deleteSetupWin), this);
 		gtk_signal_connect (GTK_OBJECT (main_window), "destroy",
 				    GTK_SIGNAL_FUNC (destroySetupWin), this);
 
@@ -146,11 +160,18 @@ XLockMore::prepareSetupWindow ()
 
 		// mode setup page
 		modePage = gtk_hbox_new (FALSE, 10);
-		GtkWidget *vb2 = gtk_vbox_new (FALSE, 10),
-			*vb3 = gtk_vbox_new (FALSE, 0);
-		setupOptions = gtk_vbox_new (FALSE, 10);
-		GtkWidget *l1 = gtk_label_new ("Mode"),
-			*l2 = gtk_label_new ("XLockMore");
+		GtkWidget *vb2 = gtk_vbox_new (FALSE, GNOME_PAD),
+			*vb3 = gtk_vbox_new (FALSE, 0),
+			*vbox = gtk_vbox_new (FALSE, 0),
+			*hbox = gtk_hbox_new (FALSE, GNOME_PAD);
+		setupOptions = gtk_vbox_new (FALSE, GNOME_PAD);
+		GtkWidget *bok = gtk_button_new_with_label (_("  Ok  ")),
+			*bapl = gtk_button_new_with_label (_(" Apply ")),
+			*bcl = gtk_button_new_with_label (_(" Cancel ")),
+			*bdf = gtk_button_new_with_label (_(" Defaults "));
+
+		GtkWidget *l1 = gtk_label_new (_(" Mode ")),
+			*l2 = gtk_label_new (_(" XLockMore "));
 		setupName = gtk_label_new ("name");
 		// setupComment = gtk_label_new ("comment");
 		// setupComment = gtk_text_new (NULL, NULL);
@@ -159,9 +180,11 @@ XLockMore::prepareSetupWindow ()
 		setupFrame = gtk_frame_new (NULL);
 		gtk_frame_set_shadow_type (GTK_FRAME (setupFrame), GTK_SHADOW_IN);
 		gtk_widget_set_usize (setupFrame, 200, 150);
-		GtkWidget *f1 = gtk_frame_new ("Options"),
-			*f2 = gtk_frame_new ("Preview"),
-			*f3 = gtk_frame_new ("Mode");
+		GtkWidget *f1 = gtk_frame_new (_(" Options ")),
+			*f2 = gtk_frame_new (_(" Preview ")),
+			*f3 = gtk_frame_new (_(" Mode ")),
+			*fb = gtk_frame_new (NULL);
+		gtk_frame_set_shadow_type (GTK_FRAME (fb), GTK_SHADOW_OUT);
 		gtk_widget_set_usize (f1, 220, -1);
 		GtkWidget *sw = gtk_scrolled_window_new (NULL, NULL);
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
@@ -182,13 +205,24 @@ XLockMore::prepareSetupWindow ()
 		gtk_box_pack_start (GTK_BOX (vb2), f3, FALSE, FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (modePage), f1, FALSE, TRUE, 0);
 		gtk_box_pack_end (GTK_BOX (modePage), vb2, FALSE, FALSE, 0);
+
 		gtk_notebook_append_page (GTK_NOTEBOOK (setupNotebook), modePage, l1);
 		gtk_notebook_append_page (GTK_NOTEBOOK (setupNotebook), hb2, l2);
-		gtk_container_border_width (GTK_CONTAINER (setupFrame), 10);
-		gtk_container_border_width (GTK_CONTAINER (sw), 10);
-		gtk_container_border_width (GTK_CONTAINER (vb3), 10);
-		gtk_container_border_width (GTK_CONTAINER (modePage), 10);
-		gtk_container_add (GTK_CONTAINER (setupWin), setupNotebook);
+
+		gtk_container_border_width (GTK_CONTAINER (setupFrame), GNOME_PAD);
+		gtk_container_border_width (GTK_CONTAINER (sw), GNOME_PAD);
+		gtk_container_border_width (GTK_CONTAINER (vb3), GNOME_PAD);
+		gtk_container_border_width (GTK_CONTAINER (modePage), GNOME_PAD);
+		gtk_box_pack_start (GTK_BOX (vbox), setupNotebook, FALSE, FALSE, 0);
+
+		gtk_container_border_width (GTK_CONTAINER (hbox), GNOME_PAD);
+		gtk_box_pack_end (GTK_BOX (hbox), bcl, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX (hbox), bapl, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX (hbox), bok, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), bdf, FALSE, FALSE, 0);
+		gtk_container_add (GTK_CONTAINER (fb), hbox);
+		gtk_box_pack_start (GTK_BOX (vbox), fb, TRUE, FALSE, 0);
+		gtk_container_add (GTK_CONTAINER (setupWin), vbox);
 
 		gtk_widget_show (hb2);
 
@@ -206,6 +240,13 @@ XLockMore::prepareSetupWindow ()
 		gtk_widget_show (setupOptions);
 		gtk_widget_show (modePage);
 		gtk_widget_show (setupNotebook);
+		gtk_widget_show (bcl);
+		gtk_widget_show (bapl);
+		gtk_widget_show (bok);
+		gtk_widget_show (bdf);
+		gtk_widget_show (hbox);
+		gtk_widget_show (fb);
+		gtk_widget_show (vbox);
 		gtk_widget_show (setupWin);
 	}
 }
@@ -218,7 +259,7 @@ XLockMore::prepareSetupWindow ()
 
 XLockMode::XLockMode (XLockMore *ss, gchar *n, gchar *c) 
      : ScreenSaverMode (ss, n, c) {
-	pars = g_list_alloc ();
+	pars = NULL;
 	parn = 0;
 }
 
@@ -231,7 +272,7 @@ XLockMode::~XLockMode ()
 void
 XLockMode::addPar (XLockModePar *par)
 {
-	g_list_append (pars, par);
+	pars = g_list_append (pars, par);
 	parn++;
 }
 
@@ -245,6 +286,12 @@ XLockMode::run (gint type, ...)
 	XLockMore *xm = (XLockMore *)parent;
 	xm->resetArg ();
 
+	GList *cur = pars;
+	while (cur) {
+		((XLockModePar *)cur->data)->setExecArgs (xm);
+		cur = cur->next;
+	}
+	
 	if (type == SS_PREVIEW || type == SS_TEST) {
 		gchar nice [32];
 		sprintf (nice, "%d", va_arg (ap, gint));
