@@ -1,7 +1,6 @@
 #!@GNOMEG@ -s
 -*- scheme -*-
 !#
-
 (define (create-itools-panel useful-options-widget
 			     extended-options-widget
 			     build-command-func)
@@ -40,10 +39,10 @@
 		   (gnome-boxed-widget (gnome-make-filled-hbox
 					#f 0
 					(gnome-boxed-widget
-					 useful-options-widget #f #f 0 'start)
-					(gnome-boxed-widget details #f #f 0 'end))
-				       #f #f 0 'start)
-		   (gnome-boxed-widget extended-options-widget #f #f 4 'start)))
+					 useful-options-widget)
+					'pack-end
+					(gnome-boxed-widget details)))
+		   (gnome-boxed-widget #f #f 4 extended-options-widget)))
 	 (hadj    (gtk-adjustment-new 0.0 0.0 0.0 0.0 0.0 0.0))
 	 (vadj    (gtk-adjustment-new 0.0 0.0 0.0 0.0 0.0 0.0))
 	 (table   (gtk-table-new 2 2 #f))
@@ -57,19 +56,20 @@
 		   
 		   (gnome-boxed-widget (gnome-make-filled-hbox
 					#f 10
-					(gnome-boxed-widget host #f #f 0 'start)
-					(gnome-boxed-widget entry #t #t 0 'start)
-					(gnome-boxed-widget stop #f #f 0 'end)
-					(gnome-boxed-widget go #f #f 0 'end))
-				       #f #f 0 'start)
+					(gnome-boxed-widget host)
+					(gnome-boxed-widget entry)
+					'pack-end
+					(gnome-boxed-widget stop)
+					(gnome-boxed-widget go)))
 		   
 		   ; Useful options and details button
 		   
-		   (gnome-boxed-widget frame #f #f 4 'start)
+		   (gnome-boxed-widget #f #f 4 frame)
 		   
 		   ; Output text widget
-		   
-		   (gnome-boxed-widget table #t #t 0 'end))))
+
+		   'pack-end
+		   (gnome-boxed-widget #t #t 0 table))))
 
     (gtk-table-attach table text 0 1 0 1 '(expand fill) '(expand fill) 0 0)
     (gtk-table-attach table hsbar 0 1 1 2 '(expand fill) '() 0 0)
@@ -371,10 +371,10 @@
      (gnome-make-filled-vbox
       #f
       3
-      (gnome-boxed-widget (gtk-label-new "GNOME Network Utilities") #f #f 0 'start)
-      (gnome-boxed-widget (gtk-label-new "Version @MYVERSION@") #f #f  0 'start)
-      (gnome-boxed-widget (gtk-label-new "Federico Mena <federico@nuclecu.unam.mx>") #f #f 0 'start)
-      (gnome-boxed-widget (gtk-label-new "Miguel de Icaza <miguel@nuclecu.unam.mx>") #f #f 0 'start)))
+      (gnome-boxed-widget (gtk-label-new "GNOME Network Utilities"))
+      (gnome-boxed-widget (gtk-label-new "Version @MYVERSION@"))
+      (gnome-boxed-widget (gtk-label-new "Federico Mena <federico@nuclecu.unam.mx>"))
+      (gnome-boxed-widget (gtk-label-new "Miguel de Icaza <miguel@nuclecu.unam.mx>"))))
     (gtk-widget-show alignment)
     alignment))
 
