@@ -2,13 +2,12 @@
 #include "applet.h"
 #include "panel.h"
 
-static Panel *panel;
-
 static void
 load_default_applets(void)
 {
-	applets_init_applet("Menu", panel, ".", 0, 0);
-	applets_init_applet("Mail check", panel, "", 550, 0);
+	applets_init_applet("Menu", ".", 0, 0);
+	applets_init_applet("Mail check", "", 550, 0);
+	applets_init_applet("Clock", "", 700, 0);
 }
 
 
@@ -17,6 +16,7 @@ init_user_applets(void)
 {
 	void *iterator;
 	char *key;
+	char *realkey;
 	char *value;
 	char *applet_name;
 	char *applet_params;
@@ -29,14 +29,14 @@ init_user_applets(void)
 	
 	while (iterator) {
 		iterator = gnome_config_iterator_next(iterator, &key, &value);
-
-		applet_params = strchr(key, ','); /* Everything after first comma is parameters to the applet */
-		if (applet_params) 
+		realkey = strchr(key, ',') + 1; /* Skip over number-for-unique-keys hack and go to applet id */
+		applet_params = strchr(realkey, ','); /* Everything after first comma is parameters to the applet */
+		if (applet_params)
 			*applet_params++ = '\0'; /* Terminate string at comma and skip over it */
 		else
 			applet_params = "";
 
-		applet_name = key;
+		applet_name = realkey;
 
 		if (sscanf(value, "%d%d", &xpos, &ypos) != 2) {
 			fprintf(stderr, "init_user_applets: using position (0, 0) for applet "
@@ -44,7 +44,9 @@ init_user_applets(void)
 
 			xpos = ypos = 0;
 		}
-		applets_init_applet(applet_name, panel, applet_params, xpos, ypos);
+		applets_init_applet(applet_name, applet_params, xpos, ypos);
+		g_free (key);
+		g_free (value);
 	}
 }
 
@@ -52,15 +54,13 @@ init_user_applets(void)
 int
 main(int argc, char **argv)
 {
-	gtk_init(&argc, &argv);
 	gnome_init(&argc, &argv);
 
-	panel = panel_new();
-
+	panel_init();
 	applets_init();
 	init_user_applets();
 
-	gtk_widget_show(panel->window);
+	gtk_widget_show(the_panel->window);
 	
 	gtk_main();
 	return 0;
