@@ -3,8 +3,6 @@
  * (C) 1997 The Free Software Foundation
  *
  * Author: Miguel de Icaza
- *
- * Exported interfaces to the GNOME Panel
  */
 
 #include <stdio.h>
@@ -18,20 +16,10 @@
 #include "../applet.h"
 #define _(String) gettext(String)
 
-struct item_config {
-	char *exec;
-	char *icon_base;
-	char *docpath;
-	char *info;
-	int  terminal;
-	char *type;
 
-	/* These are computed from icon_base */
-	char *small_icon;
-	char *transparent_icon;
-};
+#define free_if_empty(x) { if (x) g_free (x); }
 
-#define free_if_empty(x) { if (x) free (x); }
+char *gnome_folder;
 
 PanelCallback callback;
 
@@ -44,13 +32,9 @@ query (void)
 void
 activate_app_def (GtkWidget *widget, void *data)
 {
-	struct item_config *item = data;
-	char *command;
+	struct gnome_desktop_entry *item = data;
 
-	/* This is the quick hack version of this */
-	command = g_copy_strings ("(true;", item->exec, ") &", NULL);
-	system (command);
-	free (command);
+	gnome_desktop_entry_launch (item);
 }
 
 void
@@ -88,123 +72,57 @@ setup_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *thisfile)
 void
 free_app_def (GtkWidget *widget, void *data)
 {
-	struct item_config *item = data;
+	struct gnome_desktop_entry *item = data;
 
-	free_if_empty (item->exec);
-	free_if_empty (item->icon_base);
-	free_if_empty (item->docpath);
-	free_if_empty (item->info);
-	free_if_empty (item->type);
-	free_if_empty (item->small_icon);
-	free_if_empty (item->transparent_icon);
-	free (item);
+	gnome_desktop_entry_free (item);
 }
 
-int
-is_program_in_path (char *program)
+void
+add_menu_separator (GtkWidget *menu)
 {
-	static char *path;
-	static char **paths;
-	char **p;
-	char *f;
+	GtkWidget *menuitem;
 	
-	if (!path){
-		char *p;
-		int i, pc = 1;
-
-		path = strdup (getenv ("PATH"));
-		for (p = path; *p; p++)
-			if (*p == ':')
-				pc++;
-
-		paths = (char **) g_malloc (sizeof (char *) * (pc+1));
-
-		for (p = path, i = 0; i < pc; i++){
-			paths [i] = strtok (p, ":");
-			p = NULL;
-		}
-		paths [pc] = NULL;
-	}
-	p = paths;
-	while (*p){
-		f = g_concat_dir_and_file (*p, program);
-		if (g_file_exists (f)){
-			free (f);
-			return 1;
-		}
-		free (f);
-		p++;
-	}
-	return 0;
+	menuitem = gtk_menu_item_new ();
+	gtk_widget_show (menuitem);
+	gtk_container_add (GTK_CONTAINER (menu), menuitem);
 }
-	      
-struct item_config *
-load_app_def (char *file)
+
+void
+free_string (GtkWidget *widget, void *data)
 {
-	struct item_config *newitem;
-	char *prefix = g_copy_strings ("=", file, "=/Desktop Entry/", NULL);
-	char *exec_file, *try_file, *dot;
+	g_free (data);
+}
 
-	gnome_config_set_prefix (prefix);
-	free (prefix);
+void
+add_to_panel (char *applet, char *arg)
+{
+	PanelCommand cmd;
+	
+	fprintf (stderr, "Asking Panel to add [%s,%s] to the panel\n", applet, arg);
+	cmd.cmd = PANEL_CMD_NEW_APPLET;
+	cmd.params.new_applet.applet = applet;
+	cmd.params.new_applet.params  = arg;
+	(*callback) (&cmd);
+}
 
-	exec_file = gnome_config_get_string ("Exec");
-	if (!exec_file){
-		gnome_config_drop_prefix ();
-		return 0;
-	}
-	try_file = gnome_config_get_string ("TryExec");
-	if (try_file){
-		if (!is_program_in_path (try_file)){
-			free (try_file);
-			free (exec_file);
-			gnome_config_drop_prefix ();
-			return 0;
-		}
-	}
-	free (try_file);
-	newitem = g_malloc (sizeof (struct item_config));
-	newitem->exec      = exec_file;
-	newitem->icon_base = gnome_config_get_string ("Icon");
-	newitem->docpath   = gnome_config_get_string ("DocPath");
-	newitem->info      = gnome_config_get_string ("Info");
-	newitem->terminal  = gnome_config_get_int    ("Terminal");
-	newitem->type      = gnome_config_get_string ("Type");
+void
+add_app_to_panel (GtkWidget *widget, void *data)
+{
+	struct gnome_desktop_entry *ii = data;
 
-	if (newitem->icon_base && *newitem->icon_base){
-		dot = strstr (newitem->icon_base, ".xpm");
+	add_to_panel ("Launcher", ii->location);
+}
 
-		if (dot){
-			*dot = 0;
-
-			newitem->small_icon = g_copy_strings (newitem->icon_base,
-					      "-small.xpm", NULL);
-			newitem->transparent_icon = g_copy_strings (newitem->icon_base,
-					      "-transparent.xpm", NULL);
-			*dot = '.';
-		}
-
-		/* Sigh, now we need to make them local to the gnome install */
-		if (*newitem->icon_base != '/'){
-			char *s = newitem->small_icon;
-			char *t = newitem->transparent_icon;
-
-			newitem->small_icon = gnome_pixmap_file (s);
-			newitem->transparent_icon = gnome_pixmap_file (t);
-			free (s);
-			free (t);
-		}
-	} else {
-		newitem->small_icon = newitem->transparent_icon = 0;
-	}
-	gnome_config_drop_prefix ();
-	return newitem;
+void
+add_dir_to_panel (GtkWidget *widget, void *data)
+{
+	add_to_panel (query (), data);
 }
 
 GtkWidget *
-create_menu_at (GtkWidget *window, char *menudir)
+create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
 {	
-	struct item_config *item_info;
+	struct gnome_desktop_entry *item_info;
 	GtkWidget *menu;
 	struct dirent *dent;
 	struct stat s;
@@ -219,9 +137,11 @@ create_menu_at (GtkWidget *window, char *menudir)
 	menu = gtk_menu_new ();
 	
 	while ((dent = readdir (dir)) != NULL){
-		GtkWidget *menuitem, *sub, *pixmap;
-		char *thisfile = dent->d_name, *pixmap_name;
+		GtkWidget     *menuitem, *sub, *pixmap;
+		GtkSignalFunc activate_func;
+		char          *thisfile, *pixmap_name;
 
+		thisfile = dent->d_name;
 		/* Skip over . and .. */
 		if ((thisfile [0] == '.' && thisfile [1] == 0) ||
 		    (thisfile [0] == '.' && thisfile [1] == '.' && thisfile [2] == 0))
@@ -229,28 +149,57 @@ create_menu_at (GtkWidget *window, char *menudir)
 
 		filename = g_concat_dir_and_file (menudir, thisfile);
 		if (stat (filename, &s) == -1){
-			free (filename);
+			g_free (filename);
 			continue;
 		}
 
 		sub = 0;
 		item_info = 0;
 		if (S_ISDIR (s.st_mode)){
-			sub = create_menu_at (window, filename);
+			sub = create_menu_at (window, filename, create_app_menu);
 			if (!sub){
-				free (filename);
+				g_free (filename);
 				continue;
 			}
 			/* just for now */
 			pixmap_name = 0;
+
+			if (create_app_menu){
+				GtkWidget *pixmap = 0;
+				char *text;
+
+				/* FIXME: Loading the pixmap over and over is far from efficient */
+				text = g_copy_strings ("Menu: ", thisfile, NULL);
+
+				menuitem = gtk_menu_item_new ();
+				gtk_menu_prepend (GTK_MENU (sub), menuitem);
+				gtk_widget_show (menuitem);
+				
+				menuitem = gtk_menu_item_new ();
+				if (gnome_folder){
+					pixmap =gnome_create_pixmap_widget (window, menuitem, gnome_folder);
+					gtk_widget_show (pixmap);
+				}
+				setup_menuitem (menuitem, pixmap, text);
+				g_free (text);
+				text = g_strdup (filename);
+				gtk_menu_prepend (GTK_MENU (sub), menuitem);
+				gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
+						    (GtkSignalFunc) add_dir_to_panel,
+						    text);
+				gtk_signal_connect (GTK_OBJECT (menuitem), "destroy",
+						    (GtkSignalFunc) free_string,
+						    text);
+						    
+			}
 		} else {
 			if (strstr (filename, ".desktop") == 0){
-				free (filename);
+				g_free (filename);
 				continue;
 			}
-			item_info = load_app_def (filename);
+			item_info = gnome_desktop_entry_load (filename);
 			if (!item_info){
-				free (filename);
+				g_free (filename);
 				continue;
 			}
 			pixmap_name = item_info->small_icon;
@@ -270,10 +219,14 @@ create_menu_at (GtkWidget *window, char *menudir)
 		setup_menuitem (menuitem, pixmap, thisfile);
 
 		gtk_menu_append (GTK_MENU (menu), menuitem);
-		gtk_signal_connect (GTK_OBJECT (menuitem), "destroy", (GtkSignalFunc) free_app_def, item_info);
-		gtk_signal_connect (GTK_OBJECT (menuitem), "activate", (GtkSignalFunc) activate_app_def, item_info);
-		
-		free (filename);
+
+		gtk_signal_connect (GTK_OBJECT (menuitem), "destroy",
+				    (GtkSignalFunc) free_app_def, item_info);
+
+		activate_func = create_app_menu ? add_app_to_panel : activate_app_def;
+		gtk_signal_connect (GTK_OBJECT (menuitem), "activate", activate_func, item_info);
+
+		g_free (filename);
 	}
 	closedir (dir);
 
@@ -315,13 +268,102 @@ panel_reload (GtkWidget *widget, void *data)
 {
 }
 
+static void
+munge_applet_item(gpointer untrans, gpointer user_data)
+{
+	GList **list;
+	GList  *node;
+	int     pos;
+
+	list = user_data;
+
+	/* Insert applet id in alphabetical order */
+	
+	node = *list;
+	pos  = 0;
+	
+	for (pos = 0; node; node = node->next, pos++)
+		if (strcmp(_(untrans), _(node->data)) < 0)
+			break;
+	
+	*list = g_list_insert(*list, g_strdup(_(untrans)), pos);
+
+	/* We can destroy the untranslated id now */
+
+	g_free(untrans);
+}
+
+static void
+append_list_item_to_menu(gpointer data, gpointer user_data)
+{
+	GtkMenu   *menu;
+	GtkWidget *menuitem;
+
+	menu = GTK_MENU(user_data);
+	menuitem = gtk_menu_item_new();
+	setup_menuitem(menuitem, NULL, data);
+	/* FIXME: connect to "activate" signal */
+	gtk_menu_append(menu, menuitem);
+
+	g_free(data); /* We can now destroy the string */
+}
+
+static GtkWidget *
+create_applets_menu(void)
+{
+	GtkWidget    *menu;
+	GList        *list;
+	GList        *applets_list;
+	PanelCommand  cmd;
+
+	/* Get list of applet types */
+
+	cmd.cmd = PANEL_CMD_GET_APPLET_TYPES;
+	list = (*callback) (&cmd);
+
+	/* Now translate and sort them */
+
+	applets_list = NULL;
+	g_list_foreach(list, munge_applet_item, &applets_list);
+
+	/* Untranslated items have been destroyed by
+	 * munge_applet_item(), now create a menu of the translated
+	 * and sorted ones.
+	 */
+
+	g_list_free(list);
+
+	menu = gtk_menu_new();
+
+	g_list_foreach(applets_list, append_list_item_to_menu, menu);
+
+	/* Translated items have been destroyed by
+	 * append_list_item_to_menu(), now just destroy the list.
+	 */
+
+	g_list_free(applets_list);
+	return menu;
+}
+
 GtkWidget *
-create_panel_submenu (void)
+create_panel_submenu (GtkWidget *app_menu)
 {
 	GtkWidget *menu, *menuitem;
 
 	menu = gtk_menu_new ();
 	
+	menuitem = gtk_menu_item_new ();
+	setup_menuitem (menuitem, 0, _("Add to panel"));
+	gtk_menu_append (GTK_MENU (menu), menuitem);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), app_menu);
+
+	menuitem = gtk_menu_item_new ();
+	setup_menuitem (menuitem, 0, _("Add applet"));
+	gtk_menu_append (GTK_MENU (menu), menuitem);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), create_applets_menu());
+
+	add_menu_separator(menu);
+	
 	menuitem = gtk_menu_item_new ();
 	setup_menuitem (menuitem, 0, _("Configure"));
 	gtk_menu_append (GTK_MENU (menu), menuitem);
@@ -347,31 +389,22 @@ panel_logout (GtkWidget *widget, void *data)
 	PanelCommand cmd;
 
 	cmd.cmd = PANEL_CMD_QUIT;
+	(*callback) (&cmd);
 }
 
 void
-add_menu_separator (GtkWidget *menu)
-{
-	GtkWidget *menuitem;
-	
-	menuitem = gtk_menu_item_new ();
-	gtk_widget_show (menuitem);
-	gtk_container_add (GTK_CONTAINER (menu), menuitem);
-}
-
-void
-add_special_entries (GtkWidget *menu)
+add_special_entries (GtkWidget *menu, GtkWidget *app_menu)
 {
 	GtkWidget *menuitem;
 	
 	/* Panel entry */
-	menuitem = gtk_menu_item_new ();
 
 	add_menu_separator (menu);
 
+	menuitem = gtk_menu_item_new ();
 	setup_menuitem (menuitem, 0, _("Panel"));
 	gtk_menu_append (GTK_MENU (menu), menuitem);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), create_panel_submenu ());
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), create_panel_submenu (app_menu));
 
 	add_menu_separator (menu);
 	
@@ -386,10 +419,11 @@ add_special_entries (GtkWidget *menu)
 	gtk_signal_connect (GTK_OBJECT (menuitem), "activate", (GtkSignalFunc) panel_logout, 0);
 }
 
+
 GtkWidget *
 create_panel_menu (GtkWidget *window, char *menudir, int main_menu)
 {
-	GtkWidget *vbox, *button, *pixmap, *button_pixmap, *menu;
+	GtkWidget *vbox, *button, *pixmap, *button_pixmap, *menu, *app_menu;
 	static char *button_pixmap_name;
 	char *pixmap_name;
 
@@ -418,19 +452,21 @@ create_panel_menu (GtkWidget *window, char *menudir, int main_menu)
 	gtk_widget_show (pixmap);
 	gtk_widget_show (vbox);
 
-	menu = create_menu_at (window, menudir);
-	if (main_menu)
-		add_special_entries (menu);
+	menu = create_menu_at (window, menudir, 0);
+	if (main_menu){
+		app_menu = create_menu_at (window, menudir, 1);
+		add_special_entries (menu, app_menu);
+	}
 	gtk_signal_connect (GTK_OBJECT (button), "clicked", (GtkSignalFunc) activate_menu, menu);
 
-	free (pixmap_name);
+	g_free (pixmap_name);
 	return vbox;
 }
 
 GtkWidget *
 create_menu_widget (GtkWidget *window, char *arguments, char *menudir)
 {
-	GtkWidget *menu;
+	GtkWidget *menu, *add_menu;
 	int main_menu;
 	
 	main_menu = (strcmp (arguments, ".") == 0);
@@ -438,33 +474,56 @@ create_menu_widget (GtkWidget *window, char *arguments, char *menudir)
 	return menu;
 }
 
+gpointer
+panel_callback (AppletCommand *cmd)
+{
+	return NULL;
+}
+
 void
 init (PanelCallback cback, Panel *panel, char *params, int xpos, int ypos)
 {
 	GtkWidget *menu_component;
 	char *menu_base = gnome_unconditional_datadir_file ("apps");
-	char *this_menu = g_concat_dir_and_file (menu_base, params);
+	char *this_menu;
 	PanelCommand cmd;
 
 	if (!getenv ("PATH"))
 		return;
 
+	fprintf (stderr, "Menu: %s\n", params);
+	
+	if (*params == '/')
+		this_menu = strdup (params);
+	else 
+		this_menu = g_concat_dir_and_file (menu_base, params);
+
 	if (!g_file_exists (this_menu)){
-		free (menu_base);
-		free (this_menu);
+		g_free (menu_base);
+		g_free (this_menu);
 		return;
 	}
 
+	gnome_folder = gnome_unconditional_pixmap_file ("gnome-folder-small.xpm");
+	if (!g_file_exists (gnome_folder)){
+		free (gnome_folder);
+		gnome_folder = NULL;
+	}
+	
 	callback = cback;
 	menu_component = create_menu_widget (panel->window, params, this_menu);
 
+	fprintf (stderr, "REGISTRANDO: %s\n", params);
 	cmd.cmd = PANEL_CMD_REGISTER_TOY;
-	cmd.params.register_toy.applet = menu_component;
-	cmd.params.register_toy.xpos   = xpos;
-	cmd.params.register_toy.ypos   = ypos;
-	cmd.params.register_toy.flags  = 0;
+	cmd.params.register_toy.applet   = menu_component;
+	cmd.params.register_toy.callback = panel_callback;
+	cmd.params.register_toy.id       = query();
+	cmd.params.register_toy.params   = params;
+	cmd.params.register_toy.xpos     = xpos;
+	cmd.params.register_toy.ypos     = ypos;
+	cmd.params.register_toy.flags    = APPLET_HAS_PROPERTIES;
 
-	(*callback) (panel, &cmd);
+	(*callback) (&cmd);
 }
 
 #if 0
